#include <stdio.h>
#define TRUE  1
#define FALSE 0

typedef struct list_node {
	  int data;
	  struct list_node *next;
} LINKED_LIST;

LINKED_LIST* init_list();
LINKED_LIST* make_node();
int insert_node(LINKED_LIST *, int, int);
int add_node(LINKED_LIST *, int);
int delete_node(LINKED_LIST *, int);
int delete_all(LINKED_LIST *);
void print_list(LINKED_LIST *);

LINKED_LIST* init_list()
{
  LINKED_LIST* head;
  head = make_node();
  head->data = 0;
  head->next = NULL;
  return head;
}

LINKED_LIST* make_node()
{
  LINKED_LIST *temp;
  temp = (LINKED_LIST*) malloc(sizeof(struct list_node));
  if (!temp)
  {
	printf("Unable to make a new node...\n");
	return NULL;
  }
  return temp;
}

int insert_node(LINKED_LIST *list, int data, int key)
{
	LINKED_LIST *temp, *p;

	if (!list)	{
	  printf("List is not initialized....\n");
	  exit(0);
	}
	p = list;
	while (p)
	{
	  if (p->data == key)
	  {
		temp = make_node();
		temp->data = data;
		temp->next = p->next;
		p->next = temp;
		break;
	  }
	  p = p->next;
	}
	if (!p) {
	  printf("Unable to find the key node which contains %d\n", key);
	  return FALSE;
	}
	return TRUE;
}

int add_node(LINKED_LIST *list, int data)
{
	LINKED_LIST *p, *temp, *prev;
	if (!list) {
	  printf("List is not initialized....\n");
	  exit(0);
	}
	p = list;
	while (p)
	{
		prev = p;
		p = p->next;
	}
	temp = make_node();
	if (!temp)
	{
		printf("Unable to add a new node...\n");
		return FALSE;
	}
	temp->data = data;
	temp->next = NULL;
	prev->next = temp;
	return TRUE;
}

int delete_node(LINKED_LIST *list, int key)
{
	LINKED_LIST *p, *prev, *temp;

	if (!list) {
	  printf("List is not initialized....\n");
	  exit(0);
	}

	p = list;
	while (p)
	{
	  if (p->data == key)
	  {
		prev->next = p->next;
		free(p);
		break;
	  }
	  prev = p;
	  p = p->next;
	}
	if (!p) {
	  printf("Unable to find the key node for deleting...\n");
	  return FALSE;
	}
	return TRUE;
}

int delete_all(LINKED_LIST *list)
{
	LINKED_LIST *p, *prev;
	if (!list) {
		printf("List is not initialized...\n");
		exit(0);
	}
	p = list;
	while (p) {
		prev = p;
		p = p->next;
		free(prev);
	}
	return TRUE;
}

void print_list(LINKED_LIST *list)
{
	LINKED_LIST *p;
	if (!list) {
		printf("List is not initialized...\n");
		exit(0);
	}
	p = list->next;  /* skip head node */
	while (p)
	{
	  if (p->next == NULL)
		printf(" %d \n", p->data);
	  else
		printf(" %d ->", p->data);
	  p = p->next;
	}
}

int main()
{
	LINKED_LIST *mylist;

	mylist = init_list();
	add_node(mylist, 100);
	print_list(mylist);
	add_node(mylist, 200);
	print_list(mylist);
	add_node(mylist, 300);
	print_list(mylist);
	add_node(mylist, 400);
	print_list(mylist);
	add_node(mylist, 500);
	print_list(mylist);
	insert_node(mylist, 250, 200);
	print_list(mylist);
	delete_node(mylist, 300);
	print_list(mylist);

	delete_all(mylist);
	return 0;
}
